==> jaguar_session-0.1.2.tar.gz: lib/src/session_jwt_auth_header.dart <==
part of jaguar_auth.session;

/// Session manager based on JWT with Authorisation header as transport mechanism
///
/// Stores all session data in JWT token
class JwtAuthHeaderSessionManager implements SessionManager {
  /// JWT authorization context
  final JwtAuthorizationContext jwtAuthorizationContext;

  /// JWT authentication context
  final JwtAuthenticationContext jwtAuthenticationContext;

  JaguarJwtClaimSet _inClaimSet;

  /// Incoming session data parsed from the request
  final Map<String, String> _inValues = {};

  /// Outgoing session data that must be written to the response
  final Map<String, String> _outValues = {};

  /// Should the session be updated?
  bool _shouldUpdate = false;

  JwtAuthHeaderSessionManager(
      this.jwtAuthenticationContext, this.jwtAuthorizationContext);

  /// Returns a session value by given [key]
  String getInValue(String key) => _inValues[key];

  /// Update session info
  Future<Null> updateSession(Map<String, String> values) async {
    _outValues.addAll(values);
    _shouldUpdate = true;
  }

  /// Clears the session
  Future<Null> clearSession() async {
    _outValues.clear();
    _shouldUpdate = true;
  }

  /// Parses session from the given [request]
  Future<Null> parseRequest(HttpRequest request) async {
    String authHeaderStr = request.headers.value(HttpHeaders.AUTHORIZATION);

    AuthHeaderItem jwtToken =
        new AuthHeaderItem.fromHeaderBySchema(authHeaderStr, kScheme);

    //Validate token
    if (jwtToken is! AuthHeaderItem) {
      return;
    }

    _inValues.addAll(_decode(jwtToken.credentials));
  }

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate() async {
    if (_inClaimSet == null) {
      throw new UnAuthorizedError();
    }

    //Validate token
    if (!jwtAuthorizationContext.isClaimSetValid(_inClaimSet)) {
      throw new UnAuthorizedError();
    }
  }

  String get _subject => _outValues['sub'];

  /// Updates [response] with session details
  Future<Response> updateResponse(HttpRequest request, Response resp) async {
    if (!_shouldUpdate) {
      return resp;
    }

    final String oldHeader = resp.headers.value(HttpHeaders.AUTHORIZATION);
    final headers = new AuthHeaders.fromHeaderStr(oldHeader);
    headers.addItem(new AuthHeaderItem(kScheme, _encode()));
    resp.headers.set(HttpHeaders.AUTHORIZATION, headers.toString());

    return resp;
  }

  String _encode() {
    String token = jwtAuthenticationContext.encodeForSubject(_subject,
        payload: _outValues);
    return const Base64Codec.urlSafe().encode(token.codeUnits);
  }

  Map<String, dynamic> _decode(String data) {
    String tokenStr =
        new String.fromCharCodes(const Base64Codec.urlSafe().decode(data));

    //Decode and provide token
    _inClaimSet = jwtAuthorizationContext.decodeToken(tokenStr).claimSet;
    Map ret = _inClaimSet.payload ?? {};
    ret['sub'] = _inClaimSet.subject;

    return ret;
  }

  static const String kScheme = 'Bearer';
}

==> jaguar_session-0.3.0.tar.gz: lib/src/session_jwt_auth_header.dart <==
part of jaguar_auth.session;

/// Session manager based on JWT with Authorisation header as transport mechanism
///
/// Stores all session data in JWT token
class JwtAuthHeaderSessionManager implements SessionManager {
  /// JWT authorization context
  final JwtAuthorizationContext jwtAuthorizationContext;

  /// JWT authentication context
  final JwtAuthenticationContext jwtAuthenticationContext;

  JaguarJwtClaimSet _inClaimSet;

  /// Incoming session data parsed from the request
  final Map<String, String> _inValues = {};

  /// Outgoing session data that must be written to the response
  final Map<String, String> _outValues = {};

  /// Should the session be updated?
  bool _shouldUpdate = false;

  JwtAuthHeaderSessionManager(
      this.jwtAuthenticationContext, this.jwtAuthorizationContext);

  /// Returns a session value by given [key]
  String getInValue(String key) => _inValues[key];

  /// Update session info
  Future<Null> updateSession(Map<String, String> values) async {
    _outValues.addAll(values);
    _shouldUpdate = true;
  }

  /// Clears the session
  Future<Null> clearSession() async {
    _outValues.clear();
    _shouldUpdate = true;
  }

  /// Parses session from the given [request]
  Future<Null> parseRequest(Request request) async {
    String authHeaderStr = request.headers.value(HttpHeaders.AUTHORIZATION);

    AuthHeaderItem jwtToken =
        new AuthHeaderItem.fromHeaderBySchema(authHeaderStr, kScheme);

    //Validate token
    if (jwtToken is! AuthHeaderItem) {
      return;
    }

    _inValues.addAll(_decode(jwtToken.credentials));
  }

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate() async {
    if (_inClaimSet == null) {
      throw new UnAuthorizedError();
    }

    //Validate token
    if (!jwtAuthorizationContext.isClaimSetValid(_inClaimSet)) {
      throw new UnAuthorizedError();
    }
  }

  String get _subject => _outValues['sub'];

  /// Updates [response] with session details
  Future<Response> updateResponse(Request request, Response resp) async {
    if (!_shouldUpdate) {
      return resp;
    }

    final String oldHeader = resp.headers.value(HttpHeaders.AUTHORIZATION);
    final headers = new AuthHeaders.fromHeaderStr(oldHeader);
    headers.addItem(new AuthHeaderItem(kScheme, _encode()));
    resp.headers.set(HttpHeaders.AUTHORIZATION, headers.toString());

    return resp;
  }

  String _encode() {
    String token = jwtAuthenticationContext.encodeForSubject(_subject,
        payload: _outValues);
    return const Base64Codec.urlSafe().encode(token.codeUnits);
  }

  Map<String, dynamic> _decode(String data) {
    String tokenStr =
        new String.fromCharCodes(const Base64Codec.urlSafe().decode(data));

    //Decode and provide token
    _inClaimSet = jwtAuthorizationContext.decodeToken(tokenStr).claimSet;
    Map ret = _inClaimSet.payload ?? {};
    ret['sub'] = _inClaimSet.subject;

    return ret;
  }

  static const String kScheme = 'Bearer';
}
